@@ -32,7 +32,9 @@ public:
     explicit KWaylandPlugin(QObject *parent = 0);
     virtual ~KWaylandPlugin();
 
+    KWindowEffectsPrivate *createEffects() override;
     KWindowSystemPrivate *createWindowSystem() override;
+    KWindowInfoPrivate *createWindowInfo(WId window, NET::Properties properties, NET::Properties2 properties2) override;
 };
 
 #endif
